from ._conditional_dispatcher import ConditionalDispatcher
from ._decorator import decorator_with_optional_arguments

__all__ = [
    "ConditionalDispatcher",
    "decorator_with_optional_arguments",
]
